--  name: mIA_Eu.ex 
--  version: 0.1 beta_A
--  Operating system used: DOS - developed and tested on FreeDOS (http://www.freedos.org). 
--  Language: Euphoria v2.5 (http://www.rapideuphoria.com)
--  by ULTRAsmart; David Jarvis (originator), Australia. e-mail: davidj@ultrasmart.org
--  creation date: 29 Oct, 2003
--  Disclaimer: Use at your own risk. 
--              If you choose to use any part of this code, or all of the code, or the 
--		program then you do so at your own risk.
--              You have been warned.
--  Cost: FREE of charge. NO contract entered into by originator or user.
--  Intellectual rights: The originator does not claim any intellectual rights over the program, 
--                       the code, or any portion of the code.
--  Improvements or expanions: David Jarvis requests advice by email of any improvements made 
--			       to the code or any expanion of the code's use (eg for communication
--			       with other ICs). This is NOT a requirement.
--  Recognition and Thanks: 
--	    1. ports.e by Jacques Deschenes, Baie-Comeau, PQ, Canada, e-mail: desja@quebectel.com
--	    2. Euphoria mailing list community
--	    3. RDS for the Euphoria language, and the free public version
--	    4. FreeDOS community for its (free) DOS alternative		 


--  ***** Revisions:
--   		1. First official release is 0.1 beta_A. 
--   
--


--  ***** Purpose:  1. Module for IC TLC542 using parallel port to initiate
--  	            Analog sampling of specified input and read resultant data
--		    which has been converted to digital (binary) format.
--
--		    2. Easily converted to an "include" where information such as
--		    the printer port address and the input's ID are passed
--		    by the main program.


-- ***** TLC542 information:
--  TLC542 made by Texas Instruments. (Direct replacment for Motorloa MC145041)   
--  8 bit analog to digital converter; serial control; 11 inputs; CMOS technology
--  Refer http://www.ti.com for data sheet tlc542.pdf


-- ***** IC TLC542 pin (called IC4 in the program comments), LPT wire (25 pin "D" socket), analog input wire connections:
-- IC pins 1-9, and 11, 12 (A0 - A12) --> analog inputs.	pin 1 is input 1 = A0 (or address 0)
--								pin 2 is input 2 = A1 (or address 1) etc
-- IC pin 15 (CS(-)) 		--> LPT 7 (printer port wire #7 on 25 pin "D" socket)
-- IC pin 16 (DATA OUT) 	--> LPT 13
-- IC pin 17 (ADDR IN)		--> LPT 8
-- IC pin 18 (I/O CLOCK)	--> LPT 3
-- IC pin 19 (EOC)		--> LPT 10
--
-- Other pins likely connections: 
--		IC 10 (GND)	--> 0 volt (ground)
--		IC 13 (VR(-))	--> 0 volt (ground)
--		IC 14 (VR(+))	--> +5 volt
--		IC 20 (VCC)	--> +5 volt
--
-- Note: analog "input" 12 (A11 or address 11) does not connect to the "outside" world.
--       Used for test purposes only, and sends binary 1000 0000 (MSB first!) or 128 
--       regardless of reference voltages or other input voltages.
--
-- Why this IC pin --> LPT wire connection system?
-- 	Because this module was developed as part of a larger project, and the pin/wire assignments
--	may only make sense in the fuller context. Feel free to change the pin/wire assignments (and therefore
--	the relevant sections of the code).
--
-- What do I connect the other side of the analog inputs to to make a circuit so that the voltage can be measured?
--	0 volt (ground). 
--
-- Typical uses:
-- 	Where it is important to log a voltage or take some action based on the voltage.
--	Example: 	Many sensors, such as temperature sensors, send the output as a voltage. 
--			The temperature is calculated from a formula which includes the output voltage
--			



-- +++++++++++++++++++++++++ INCLUDES +++++++++++++++++++++++++++++++
-- Euphoria general "includes"              
include Machine.e
include Graphics.e	-- ===>*USER* may remove. Required only if you want to print something
include File.e	

-- Special "Includes"
include Ports.e    -- used to read and write to the parallel port



-- +++++++++++++++++++++++ DECLARATIONS +++++++++++++++++++++++++++++++++++++++++++++++

object keybrd		-- ===>*USER* may remove (and other relevant code). 
			-- Enables break out of any infinite looping by pressing CTL-C. 


atom LPT1Addr		-- Para Port address (in hex). Usually #378, but other possible values include #278, #3BC
LPT1Addr = #378		-- ===>*USER* defines the port being used to comunicate with the IC TLC542 by inserting the correct address

integer IA_ID		-- Euphoria form of the ID of the input on the IC

sequence IAo     	-- hold each analog input's output data. Normalised values ranging fromn 0 = min to 1 = max. 
IAo = repeat(0,12)	-- 12 inputs -- Only 11 available to use, but 12th (always = 128; normalised = 0.5) is internal and useful as a check

atom I_out		-- The determined output data for the selected analog input. 
			-- The data for Input 1 is placed in IAo[1], for Input 2 is placed in IAo[2] etc

-- ======================================================================================

-- ***************************************************************************************
-- ***************************************************************************************

-- PROGRAM -------------------------------------------------------------------------------

-- *********** below may be set up in the "main" control program if this is converted to an "include"
integer IAP		-- stands for "In Put Analog". 
	IAP = 26	-- Anticipate this to be declared as a global constant with value (say 26) 
			-- in the "main" program if this becomes an "include".
			-- Not really needed in this particular example, but if a different type of
			-- input was also to be monitored by the control program, then it's important
			-- to have a way of giving each type a different name.
	
sequence module 	-- each analog input is represented by sequence called a "module"
module = {IAP, 12}	-- ===>*USER* defines which input is to be monitored by changing the
			--            value of module[2] prior to running the program
-- **************************************************************************************


sequence bin_rep_of_input, voltage_bin, IAP_address_bin, zero_byte, temp_s1
integer int_rep_of_input, voltage_int, IAP_address_int, temp_i1
atom voltage, MaxVolts

 
zero_byte = repeat(0,8)	 



allow_break(1)                  -- ===>*USER* may remove aloing with other relevant code.  Allows CTL-C to stop program 



-- initialise sequences, atoms etc needed
MaxVolts = 255				-- 8 bit system - max can count to is 255. 
					-- So if the max voltage able to be measured by the IC is (say) 5 volts, then
					-- if 5 volts was placed on the input, the IC would output
					-- binary (1111 1111) which  equals decimal integer 255

bin_rep_of_input = repeat(5,8)		-- use "5" rather than 1 or 0 so can test to see if changes have occurred
voltage_bin = repeat(5,8)		-- use "5" rather than 1 or 0 so can test to see if changes have occurred



-- Get Output module number, and calc the input's address
IA_ID = module[2]					-- input number 
IAP_address_int = IA_ID - 1				-- input address (as used by the IC).
							-- The IC numbers inputs from 0 ->11 (vs 1->12 decimal as used in Euphoria),
							-- so the IC's input address is the Inputs ID minus 1

IAP_address_bin = int_to_bits(IAP_address_int,4)	-- convert input's address to binary. Only need 4 bits



-- deselect ADC
-- pin2=0, clock=0, pin4=0, pin5=0, pin6=0, IC4_deselect=1, IC4_input_address=0, pin9=0 -- base0
Output(32, LPT1Addr) 	-- decimal 32 = 00000100 (LSB first) -- IC4_deselect=1


-- *************************************************************************************

-- loop twice. 
-- 1st loop pushes the address of input to be sampled into the IC
-- 2nd loop reads the digital data output
for j = 1 to 2 do

	-- select ADC
	-- pin2=0, clock=clock_lo=0, pin4=0, pin5=0, pin6=0, IC4_deselect=0, IC4_input_address=0, pin9=0 -- base0
	Output(0, LPT1Addr) 	-- decimal 0 = 00000000 (LSB first) -- IC4 deselect set to 0

	-- start SHIFT OUT of the last conversion process' results 
	-- FIRST time this just flushes the cache in preparation for the new results to be saved.
	-- SECOND time, allows transmission of the results from the NEW sampling

	-- get sampling MSB (bit 7) result.
	-- after the deselect/select process, the MSB of the last data conversion appears on the output (connected to wire 10 of LPT)
	-- get the MSB by reading pin 13 (base0 + 1, bit 4 = positon 5 in bit sequence = 10 decimal)
	int_rep_of_input = Input(LPT1Addr+1)			-- read the printer port base0+1
	bin_rep_of_input = int_to_bits(int_rep_of_input,8)	-- convert the result to bits so can check on bit4
	voltage_bin[8] = bin_rep_of_input[5]			-- place data bit into MSB position of sequence storing voltage byte


	-- send first four bits (MSB first) of the input address on the first 4 rising edges of the CLOCK,
	-- and read the next 4 most significant bits of sampling result data (output) on the falling edges of the clock
	for i = 1 to 4 do

		-- send IP addresses data bits, one at a time
		temp_s1 = zero_byte			-- set up all zeros in byte - so can send a full byte as a Output command
		temp_s1[7] = IAP_address_bin[5-i]	-- put in the next bit of the input's address (MSB first) 
		temp_i1 = bits_to_int(temp_s1)		-- convert to integer so can write to port
		Output(temp_i1, LPT1Addr) 		-- output

		-- send clock to hi while keeping address in registers
		Output(2+temp_i1, LPT1Addr) 	-- decimal 2 = 01000000 (LSB first) -- note als keeps IC4 deselect set to 0 - ie IC4 remains selected

		
		-- send clock to lo while keeping address in registers
		Output(0+temp_i1, LPT1Addr) 	-- decimal 0 = 00000000 (LSB first) -- note als keeps IC4 deselect set to 0 - ie IC4 remains selected

		-- read the next i of 4 most significant bits of sampling result data
		int_rep_of_input = Input(LPT1Addr+1)			-- read the printer port base0+1
		bin_rep_of_input = int_to_bits(int_rep_of_input,8)	-- convert the result to bits so can check on bit4 (=position5 in the sequence of bits)
		voltage_bin[8-i] = bin_rep_of_input[5]			-- place next data bit into the next MSB position of sequence storing voltage byte
	end for

	-- sampling begins now after the fourth falling edge - last rep of the "for i = 1 to 4" above

	-- send 3 more clock pulses to shift out and capture the last 3 bits of data on the falling edge of the pulse
	for i = 1 to 3 do
		-- send clock hi
		Output(2, LPT1Addr) 	-- decimal 2 = 01000000 (LSB first) -- note als keeps IC4 deselect set to 0 - ie IC4 remains selected
		-- send clock to lo 
		Output(0, LPT1Addr) 	-- decimal 0 = 00000000 (LSB first) -- note als keeps IC4 deselect set to 0 - ie IC4 remains selected

		-- read the next i of 3 most significant bits of sampling result data
		int_rep_of_input = Input(LPT1Addr+1)			-- read the printer port base0+1
		bin_rep_of_input = int_to_bits(int_rep_of_input,8)	-- convert the result to bits so can check on bit4
		voltage_bin[4-i] = bin_rep_of_input[5]			-- place next data bit into the next MSB position of sequence storing voltage byte
	end for

	-- apply 8th clock pulse. On its falling edge, expect IC4's EOC output to go low (chip's pin 19, connected to Printer wire 10)
	-- the EOC lo signals still sampling and converting data.
		-- send clock hi
		Output(2, LPT1Addr) 	-- decimal 2 = 01000000 (LSB first) -- note also keeps IC4 deselect set to 0 - ie IC4 remains selected
		-- send clock to lo 
		Output(0, LPT1Addr) 	-- decimal 0 = 00000000 (LSB first) -- note also keeps IC4 deselect set to 0 - ie IC4 remains selected

	-- deselect chip (while keeping clock low) so as not to interrupt the conversion process with any "noise" on the system
	-- pin2=0, clock=0, pin4=0, pin5=0, pin6=0, IC4_deselect=1, IC4_input_address=0, pin9=0 -- base0
	Output(32, LPT1Addr) 	-- decimal 32 = 00000100 (LSB first) 	-- IC4_deselect=1

	-- watch to find when the EOC output goes high again to signal conversion completed
	while 1 do
		int_rep_of_input = Input(LPT1Addr+1)			-- read the printer port base0+1
		bin_rep_of_input = int_to_bits(int_rep_of_input,8)
			if bin_rep_of_input[7] = 1 then			-- bit 6 (position 7 on sequence) = 1 all others =0 ie EOC has gone high
				exit					-- break out of while loop
			end if
		keybrd = get_key()      -- ===>*USER* may remove along with other relevant code.  Allows CTL-C to stop program.  
	end while
end for


-- now have the full digital value of the input's voltage stored in sequence voltage_bin (LSB first)
-- calculate integer represesntation of voltage input value 
-- for 8 bit resolution (or 1 in 256) if MAX = 1 (or 100%), then 1/256 = 0.004 (or 0.4%) is best possible resolution!
-- so for max range of 5 volt (0->5 volt), best resolution is 0.0195 volt (say 20mV)

voltage_int = bits_to_int(voltage_bin)			-- result should be between 0 and 255
voltage = voltage_int/255				-- normalise to get value between 0 and 1.0



-- Print normalised analog voltage. Can be returned to the Main program if this was an "include".
I_out = voltage
? I_out




-- --------------------------------------------END-----------------------------------------------------------

